/*

auths-securid.h

  Authors: Graeme Ahokas <gahokas@ssh.com>

  Copyright (C) 2000 SSH Communications Security Corp, Helsinki, Finland
  All rights reserved.

  SecurID authentication, server side
*/

#ifndef AUTHS_SECURID_H
#define AUTHS_SECURID_H

#ifdef SSH_SERVER_WITH_SECURID

#include "sshuser.h"


/* Function used to represent an authentication method.  This function
   performs all processing by the authentication method.  An authentication
   method is policy-independent.
     `user'         user name from the (original) authentication request
     `uc'           SshUser object corresponding to `user' or NULL if invalid
     `packet'       method-specific remaining part of the packet
     `session_id'   session identifier
     `session_id_len' length of session identifier
     `state_placeholder' place to store context data between packets
     `longtime_placeholder'  can hold data between authentications
                    (this is per-method)
     `completion_proc'  callback, that notifies the authentication protocol
                        on the result of the authentication method.
     `completion_context' context for `completion_proc'
     `method_context'   passed to the method function */
void ssh_server_auth_securid(SshAuthServerOperation op,
                            const char *user,
                            SshUser uc,
                            SshBuffer packet,
                            const unsigned char *session_id,
                            size_t session_id_len,
                            void **state_placeholder,
                            void **longtime_placeholder,
                            SshAuthServerCompletionProc completion_proc,
                            void *completion_context,
                            void *method_context);

#endif /* SSH_SERVER_WITH_SECURID */
#endif /* AUTHS_SECURID_H */
