/*

  Author: Antti Huima <huima@ssh.fi>

  Copyright (C) 1996, 2001 SSH Communications Security Oy, Espoo, Finland
  All rights reserved.

  Created: Mon May  6 00:13:23 1996 [huima]

  Functions for generating primes.

*/

/*
 * $Id: sshgenmp.h,v 1.2 2001/07/26 10:05:13 santtu Exp $
 * $Log: sshgenmp.h,v $ * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * 
 * $EndLog$
 */

#ifndef GENMP_H
#define GENMP_H

/* Generates a random integer of the desired number of bits. */

void ssh_mp_random_integer(SshMPInt ret, unsigned int bits);

/* Makes and returns a random pseudo prime of the desired number of bits.
   Note that the random number generator must be initialized properly
   before using this.

   The generated prime will have the highest bit set, and will have
   the two lowest bits set.

   Primality is tested with Miller-Rabin test, ret thus having
   probability about 1 - 2^(-50) (or more) of being a true prime.
   */
void ssh_mp_random_prime(SshMPInt ret,unsigned int bits);

/* Generate a random prime within the [min, max] interval. We observe that
   the process can just choose a random number modulo (max - min) and
   then start from there. If it goes beyond max-1 then it cycles.
   */
void ssh_mp_random_prime_within_interval(SshMPInt ret,
                                         SshMPInt min, SshMPInt max);

/* Similar to the ssh_mp_random_prime, except that the 'strong' pseudo
   prime is returned. Uses method described in P1363 working draft.

   'big_bits' tells how many bits are in the 'prime' and 'small_bits' how
   many bits in the 'div'. Note that 'prime' - 1 = 0 mod 'div'.

   This method generates good primes for RSA or other factorization based
   cryptosystems. For discrete log based systems this isn't exactly
   neccessary.
   */

void ssh_mp_strong_p1363_random_prime(SshMPInt prime, SshMPInt div,
                                      int big_bits, int small_bits);

/* Generate a strong random prime, where 'prime' = 'order' * u + 1. Similar
   to the P1363 method but takes less time and is almost as 'strong'. The
   P1363 strong primes satisfy some other facts, but in practice these
   primes seem as good with discrete log based cryptosystems. */

void ssh_mp_random_strong_prime(SshMPInt prime,
                                SshMPInt order,
                                int prime_bits, int order_bits);

/* Modular invert with positive results. */

int ssh_mp_mod_invert(SshMPInt op_dest, const SshMPInt op_src,
                      const SshMPInt modulo);

/* Random number with special modulus */

void ssh_mp_mod_random(SshMPInt op, const SshMPInt modulo);

/* Generate a random integer with entropy at most _bits_ bits. The atmost,
   means that the actual number of bits depends whether the modulus is
   smaller in bits than the _bits_.  */
void ssh_mp_mod_random_entropy(SshMPInt op, const SshMPInt modulo,
                               unsigned int bits);


#if 0
/* Lucas functions */

void ssh_mp_reduced_lucas(SshMPInt op_dest, const SshMPInt op_e,
                          const SshMPInt op_p, const SshMPInt op_n);

void ssh_mp_lucas(SshMPInt op_dest, const SshMPInt op_src1,
                  const SshMPInt op_src2,
                  const SshMPInt k, const SshMPInt modulo);

/* Modular square root */

int ssh_mp_mod_sqrt(SshMPInt op_dest, const SshMPInt op_src,
                    const SshMPInt modulo);
#endif

/* Check the MOV condition, for elliptic curves */

Boolean ssh_mp_mov_condition(const SshMPInt b,
                             const SshMPInt q, const SshMPInt r);

/* Check whether op_src is of order op_ord mod modulo */

int ssh_mp_is_order(const SshMPInt op_ord, const SshMPInt op_src,
                    const SshMPInt modulo);

/* Find a random generator of order 'order' modulo 'modulo'. */

Boolean ssh_mp_random_generator(SshMPInt g, SshMPInt order, SshMPInt modulo);
#endif /* GENMP_H */
